@@ -26,8 +26,8 @@
    might be covered by the GNU General Public License.
 -------------------------------------------------------------------------*/
 
-#ifndef __SDCC_SETJMP_H
-#define __SDCC_SETJMP_H
+#ifndef __STDC_VERSION_SETJMP_H__
+#define __STDC_VERSION_SETJMP_H__ __STDC_VERSION__
 
 #define SP_SIZE		1
 
@@ -51,13 +51,28 @@
 #define RET_SIZE	2
 #endif
 
-#if defined (__SDCC_z80) || defined (__SDCC_z180) || defined (__SDCC_r2k) || defined (__SDCC_r3ka) || defined (__SDCC_tlcs90)
+#if defined (__SDCC_z80) || defined (__SDCC_z180) || defined (__SDCC_r2k) || defined (__SDCC_r2ka) || defined (__SDCC_r3ka) || defined (__SDCC_tlcs90) || defined (__SDCC_ez80_z80) || defined (__SDCC_z80n) || defined(__SDCC_r800)
 typedef unsigned char jmp_buf[6]; /* 2 for the stack pointer, 2 for the return address, 2 for the frame pointer. */
 #elif defined (__SDCC_ds390) || defined (__SDCC_stm8) && defined (__SDCC_MODEL_LARGE)
 typedef unsigned char jmp_buf[5]; /* 2 for the stack pointer, 3 for the return address. */
-#elif defined (__SDCC_stm8) || defined (__SDCC_gbz80) || defined (__SDCC_hc08) || defined (__SDCC_s08)
+#elif defined (__SDCC_stm8) || defined (__SDCC_sm83) || defined (__SDCC_hc08) || defined (__SDCC_s08) || defined (__SDCC_f8)
 typedef unsigned char jmp_buf[4]; /* 2 for the stack pointer, 2 for the return address. */
-#else
+#elif defined (__SDCC_pdk13) || defined (__SDCC_pdk14) || defined (__SDCC_pdk15) || defined(__SDCC_mos6502) || defined(__SDCC_mos65c02)
+typedef unsigned char jmp_buf[3]; /* 1 for the stack pointer, 2 for the return address. */
+#elif defined (__SDCC_pic16)
+/*
+ * return address - 3 (TOS)
+ * return address stack index - 1 (STKPTR)
+ * stack pointer - 1 (small) / 2 (large) (FSR1)
+ * frame pointer - 1 (small) / 2 (large) (FSR2)
+ */
+    #if defined(__SDCC_MODEL_LARGE)
+        #define __SETJMP_H_STACK_PTR_SIZE 2
+    #else
+        #define __SETJMP_H_STACK_PTR_SIZE 1
+    #endif
+typedef unsigned char jmp_buf[2 * __SETJMP_H_STACK_PTR_SIZE + 4];
+#elif defined (__SDCC_mcs51)
 typedef unsigned char jmp_buf[RET_SIZE + SP_SIZE + BP_SIZE + SPX_SIZE + BPX_SIZE];
 #endif
 
