@@ -362,7 +362,7 @@ gen_xop_arith(int rf, int op, int t1, int t2, int v1, int v2, struct expr *e1, s
     {
       outab(0xf8 + v1);
       outab(op);
-      outab(v2);
+      outrb(e2, 0);
       return 1;
     }
 
@@ -417,7 +417,7 @@ gen_xop_arith(int rf, int op, int t1, int t2, int v1, int v2, struct expr *e1, s
           outab(0xef);
           outab(v1 & 0xFF);
           outab(op);
-          outab(v2);
+          outrb(e2, 0);
           return 1;
         }
       else
@@ -425,7 +425,7 @@ gen_xop_arith(int rf, int op, int t1, int t2, int v1, int v2, struct expr *e1, s
           outab(0xeb);
           outrw(e1, 0);
           outab(op);
-          outab(v2);
+          outrb(e2, 0);
           return 1;
         }
     }
@@ -442,9 +442,9 @@ gen_xop_arith(int rf, int op, int t1, int t2, int v1, int v2, struct expr *e1, s
 void
 machine(struct mne *mp)
 {
-  int op, t1, t2;
+  int op, t1, t2, t3;
   struct expr e1, e2;
-  int rf, v1, v2;
+  int rf, v1, v2, v3;
 
   clrexpr(&e1);
   clrexpr(&e2);
@@ -474,6 +474,32 @@ machine(struct mne *mp)
 
       //////////////////////////////////////////////////////////////////////
 
+    case S_LDA:
+      t1 = addr(&e1);
+      v1 = (int) e1.e_addr;
+      comma(1);
+      t2 = addr(&e2);
+      v2 = (int) e2.e_addr;
+      comma(1);
+      t3 = addr(&e2);
+      v3 = (int) e2.e_addr;
+
+      if (t1 == S_R16 && (t2 == S_R16) && (v2==IX || v2==IY || v2==SP) && t3 == S_IMMED)
+        {
+          outab(op + v2 - IX);
+          outrb(&e2, R_SGND);
+          outab(0x38 + (v1 & 0x0F));
+          break;
+        }
+      if (t1 == S_R16 && (t2 == S_R16) && (v2==HL) && t3 == S_R8 && v3 == A)
+        {
+          outab(0xF7);
+          outab(0x38 + (v1 & 0x0F));
+          break;
+        }
+      aerr();
+      break;
+
     case S_INCX:
     case S_DECX:
       t1 = addr(&e1);
@@ -1002,6 +1028,7 @@ machine(struct mne *mp)
     case S_BIT:
     case S_RES:
     case S_SET:
+    case S_TSET:
       t1 = addr(&e1);
       v1 = (int) e1.e_addr;
       comma(1);
